use crate::jira_config::JiraConfig;
use gray_matter::engine::TOML;
use gray_matter::Matter;
use libjirac::client::commands::{
    IssueCreateCommand, IssueTransitionsCommand, IssueTransitionsUpdateCommand,
};
use libjirac::client::JiraClient;
use libjirac::entities::issue_request::*;
use serde::{Deserialize, Serialize};
use std::collections::HashMap;
use std::fs;
use std::io::Write;
use std::path::PathBuf;
use tempfile::NamedTempFile;

#[derive(Debug, Deserialize, Serialize)]
struct IssueMetadata {
    #[serde(skip_serializing_if = "Option::is_none")]
    status: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    project: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    assignee: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    parent: Option<String>,
    #[serde(flatten)]
    extra: HashMap<String, serde_json::Value>,
}

fn get_issue_template(
    issue_metadata: &IssueMetadata,
) -> Result<String, Box<dyn std::error::Error>> {
    let matter = toml::to_string(issue_metadata)?;
    let has_project = issue_metadata.project.is_some();

    let template = format!(
        r#"---
# The issue can contain the following properties:
# - status
# - project (required)
# - assignee
# - parent
# - labels
# - priority

{}
{}
---
# Title

[description]
"#,
        if has_project { "" } else { r#"project = """# },
        matter,
    );

    Ok(template)
}

fn get_editor() -> String {
    std::env::var("EDITOR")
        .or_else(|_| std::env::var("VISUAL"))
        .unwrap_or_else(|_| "vi".to_string())
}

fn parse_markdown(
    content: &str,
) -> Result<(String, String, IssueMetadata), Box<dyn std::error::Error>> {
    let matter = Matter::<TOML>::new();
    let result = matter.parse_with_struct::<IssueMetadata>(content);

    let (metadata, content) = match result {
        Some(x) => (x.data, x.content),
        None => {
            eprintln!("unexpected markdown content found");
            std::process::exit(1);
        }
    };

    // let metadata = result.data
    //     .and_then(|d| toml::from_str(d).ok())
    //     .unwrap_or_else(|| IssueMetadata {
    //         status: None,
    //         project: None,
    //         assignee: None,
    //         parent: None,
    //         extra: HashMap::new(),
    //     });
    //
    // let content = result.content;
    let mut lines = content.lines();

    // Get first non-empty line as title
    let title = lines
        .by_ref()
        .find(|line| !line.trim().is_empty())
        .unwrap_or("")
        .trim_start_matches('#')
        .trim()
        .to_string();

    // Rest is description
    let description = lines.collect::<Vec<&str>>().join("\n");

    Ok((title, description, metadata))
}

fn create_temp_markdown(project_key: Option<String>) -> Result<String, Box<dyn std::error::Error>> {
    let mut temp_file = NamedTempFile::new()?;

    let metadata = IssueMetadata {
        status: None,
        project: project_key,
        assignee: None,
        parent: None,
        extra: Default::default(),
    };

    let template = get_issue_template(&metadata)?;

    temp_file.write_all(template.as_bytes())?;
    temp_file.flush()?;

    let editor = get_editor();
    let status = std::process::Command::new(editor)
        .arg(temp_file.path())
        .status()?;

    if !status.success() {
        return Err("Editor exited with non-zero status".into());
    }

    // Read the modified content
    let content = fs::read_to_string(temp_file.path())?;

    // temp_file will be automatically deleted when it goes out of scope
    Ok(content)
}
async fn create_jira_issue(
    config: &JiraConfig,
    project_key: &str,
    title: &str,
    description: &str,
    metadata: &IssueMetadata,
) -> Result<IssueCreateResponse, Box<dyn std::error::Error>> {
    let client = JiraClient::from(config);

    let fields = IssueFields {
        project: Project {
            key: project_key.to_string(),
        },
        summary: title.to_string(),
        description: description.to_string(),
        issue_type: IssueType {
            name: "Task".to_string(),
        },
        assignee: None,
        status: None,
    };

    let mut update = None;

    // Add parent if specified
    if let Some(parent_key) = &metadata.parent {
        update = Some(IssueUpdate {
            parent: Some(vec![ParentUpdate {
                add: Parent {
                    key: parent_key.clone(),
                },
            }]),
        });
    }

    let issue = IssueCreateRequest { fields, update };

    let create_cmd = IssueCreateCommand::new(issue);
    let issue_response = client.exec(create_cmd).await?;

    // Update status if specified (requires a separate API call)
    if let Some(status) = &metadata.status {
        update_issue_status(config, &issue_response.key, status).await?;
    }

    // Update assignee if specified
    if let Some(_assignee) = &metadata.assignee {
        // Need to look up users by username, because the api wants an account id
        // fields.assignee = Some(Assignee {
        //     id: assignee.clone(),
        // });
    }

    Ok(issue_response)
}

async fn update_issue_status(
    config: &JiraConfig,
    issue_key: &str,
    status: &str,
) -> Result<(), Box<dyn std::error::Error>> {
    let client = JiraClient::from(config);

    let transitions_cmd = IssueTransitionsCommand::new(issue_key);
    let transitions = client.exec(transitions_cmd).await?;

    // Find the transition ID for the desired status
    let transition_id = transitions
        .transitions
        .iter()
        .find_map(|t| {
            if t.to.name.eq_ignore_ascii_case(status) {
                Some(t.id.clone())
            } else {
                None
            }
        })
        .ok_or_else(|| format!("No transition found for status: {}", status))?;

    let transition_cmd = IssueTransitionsUpdateCommand::new(issue_key, &transition_id);
    match client.exec(transition_cmd).await {
        Ok(x) => Ok(x),
        Err(reason) => Err(reason.into()),
    }
}

pub async fn create(
    project: Option<String>,
    open: bool,
    markdown_file: Option<PathBuf>,
) -> Result<(), Box<dyn std::error::Error>> {
    let config = JiraConfig::load().map_err(|e| format!("Configuration error: {}", e))?;

    let content = match markdown_file {
        Some(path) => {
            fs::read_to_string(&path).map_err(|e| format!("Failed to read file: {}", e))?
        }
        None => {
            eprintln!("No markdown file specified. Opening editor...");
            create_temp_markdown(project)?
        }
    };

    if content.trim().is_empty() {
        return Err("Empty content. Aborting issue creation.".into());
    }

    let (title, description, metadata) = parse_markdown(&content)?;

    let selected_project = match metadata.project.clone() {
        Some(x) => x.to_uppercase(),
        None => {
            eprintln!("No project set...");
            std::process::exit(1);
        }
    };

    if selected_project.is_empty() {
        eprintln!("No project set...");
        std::process::exit(1);
    }

    // Confirm creation
    println!("\nAbout to create an issue:");
    println!("Project: {}", selected_project);
    if let Some(status) = &metadata.status {
        println!("Status: {}", status);
    }
    if let Some(assignee) = &metadata.assignee {
        println!("Assignee: {}", assignee);
    }
    if let Some(parent) = &metadata.parent {
        println!("Parent: {}", parent);
    }
    println!("Title: {}", title);

    println!("Description:\n{}", description);
    println!("\nPress Enter to continue or Ctrl+C to abort");

    let mut input = String::new();
    std::io::stdin().read_line(&mut input)?;

    let response =
        create_jira_issue(&config, &selected_project, &title, &description, &metadata).await?;

    let url = format!("{}/browse/{}", config.url, response.key);

    println!("Successfully created issue: {}", response.key);
    println!("URL: {}", url);

    if open {
        open::that(url)?;
    }

    Ok(())
}
