use crossterm::style::Stylize;
use libjirac::entities::doc::{Doc, DocMedia, DocNode, DocTextMarker, DocTextNode};
use std::fmt::Write;

fn render_doc_node(f: &mut String, node: &DocNode) -> Result<(), Box<dyn std::error::Error>> {
    match node {
        DocNode::Paragraph(x) => {
            for node in x {
                render_text_node(f, node)?;
            }
            writeln!(f)?;
        }
        DocNode::BulletList(x) => {
            for node in x {
                write!(f, "* ")?;
                render_doc_node(f, node)?;
            }
        }
        DocNode::ListItem(x) => {
            for node in x {
                render_doc_node(f, node)?;
            }
        }
        DocNode::OrderedList(x) => {
            let mut i = 1;
            for node in x {
                write!(f, "{}. ", i)?;
                i += 1;
                render_doc_node(f, node)?;
            }
        }
        DocNode::Rule => {
            writeln!(f, "---")?;
        }
        DocNode::Heading(x) => {
            for node in x {
                render_text_node(f, node)?;
            }
        }
        DocNode::MediaSingle(x) => {
            for node in x {
                render_text_markets(f, node)?;
            }
        }
    }

    Ok(())
}

fn render_text_node(f: &mut String, node: &DocTextNode) -> Result<(), Box<dyn std::error::Error>> {
    match node {
        DocTextNode::Text(x) => {
            write!(f, "{}", x.text)?;
        }
        DocTextNode::HardBreak => {
            writeln!(f)?;
        }
        DocTextNode::InlineCard(x) => {
            writeln!(f, "{}", x.attrs.url)?;
        }
        DocTextNode::Mention(x) => {
            write!(f, "{}", x.attrs.text.clone().green())?;
        }
    }

    Ok(())
}

fn render_text_markets(
    f: &mut String,
    node: &DocTextMarker,
) -> Result<(), Box<dyn std::error::Error>> {
    match node {
        DocTextMarker::Link(x) => {
            write!(f, "{}", x.href)?;
        }
        DocTextMarker::Strong => {
            // no-op
        }
        DocTextMarker::Code => {
            // no-op
        }
        DocTextMarker::Media(x) => {
            render_media(f, x)?;
        }
    }

    Ok(())
}

fn render_media(f: &mut String, node: &DocMedia) -> Result<(), Box<dyn std::error::Error>> {
    match node {
        DocMedia::File(x) => {
            writeln!(f, "[file:{}:{}]", x.id, x.alt)?;
        }
    }

    Ok(())
}

pub fn render_doc(doc: Doc) -> Result<String, Box<dyn std::error::Error>> {
    let mut f = String::new();

    for node in doc.content {
        render_doc_node(&mut f, &node)?;
    }

    Ok(f)
}
